@@ -38,6 +38,7 @@ import json
 import yaml
 import sys
 import subprocess
+import io
 
 from enum import Enum
 from lxml import etree
@@ -117,8 +118,8 @@ class FilterBase(object, metaclass=TrackSubClasses):
             filter_spec = [dict([filter_kind.split(':', 1)]) if ':' in filter_kind else filter_kind
                            for filter_kind in filter_spec.split(',')]
 
-            logger.warn('String-based filter definitions (%s) are deprecated, please convert to dict-style:\n\n%s',
-                        old_filter_spec, yaml.dump(filter_spec, default_flow_style=False))
+            logger.warning('String-based filter definitions (%s) are deprecated, please convert to dict-style (see https://urlwatch.readthedocs.io/en/latest/deprecated.html):\n\n%s',
+                           old_filter_spec, yaml.dump(filter_spec, default_flow_style=False))
 
         if isinstance(filter_spec, list):
             for item in filter_spec:
@@ -145,9 +146,20 @@ class FilterBase(object, metaclass=TrackSubClasses):
         return filtercls(state.job, state).filter(data, subfilter)
 
     @classmethod
-    def is_bytes_filter(cls, filter):
-        return (filter in [name for name, class_ in FilterBase.__subclasses__.items()
-                           if getattr(class_, '__uses_bytes__', False)])
+    def filter_chain_needs_bytes(cls, filter):
+        # If the first filter is a bytes filter, return content in bytes instead of
+        # in unicode as that's what's required by the library used by that filter
+        first_filter = next(cls.normalize_filter_list(filter), None)
+        if first_filter is not None:
+            filter_kind, subfilter = first_filter
+            return cls.is_bytes_filter_kind(filter_kind)
+
+        return False
+
+    @classmethod
+    def is_bytes_filter_kind(cls, filter_kind):
+        return (filter_kind in [name for name, class_ in cls.__subclasses__.items()
+                                if getattr(class_, '__uses_bytes__', False)])
 
     def match(self):
         return False
@@ -211,7 +223,7 @@ class LegacyHooksPyFilter(FilterBase):
                 result = data
             return result
         except Exception as e:
-            logger.warn('Could not apply legacy hooks filter: %s', e)
+            logger.warning('Could not apply legacy hooks filter: %s', e)
             return data
 
 
@@ -301,7 +313,6 @@ class Pdf2TextFilter(FilterBase):
             raise ValueError('The pdf2text filter needs bytes input (is it the first filter?)')
 
         import pdftotext
-        import io
         return '\n\n'.join(pdftotext.PDF(io.BytesIO(data), password=subfilter.get('password', '')))
 
 
@@ -547,26 +558,18 @@ class LxmlParser:
 
     def __init__(self, filter_kind, subfilter, expr_key):
         self.filter_kind = filter_kind
-        if subfilter is None:
+        if expr_key not in subfilter:
             raise ValueError('Need %s for filtering' % (self.EXPR_NAMES[filter_kind],))
-        if isinstance(subfilter, str):
-            self.expression = subfilter
-            self.method = 'html'
-            self.exclude = None
-            self.namespaces = None
-        elif isinstance(subfilter, dict):
-            if expr_key not in subfilter:
-                raise ValueError('Need %s for filtering' % (self.EXPR_NAMES[filter_kind],))
-            self.expression = subfilter[expr_key]
-            self.method = subfilter.get('method', 'html')
-            self.exclude = subfilter.get('exclude')
-            self.namespaces = subfilter.get('namespaces')
-            if self.method not in ('html', 'xml'):
-                raise ValueError('%s method must be "html" or "xml", got %r' % (filter_kind, self.method))
-            if self.method == 'html' and self.namespaces is not None:
-                raise ValueError('Namespace prefixes only supported with "xml" method.')
-        else:
-            raise ValueError('%s subfilter must be a string or dict' % (filter_kind,))
+        self.expression = subfilter[expr_key]
+        self.method = subfilter.get('method', 'html')
+        self.exclude = subfilter.get('exclude')
+        self.namespaces = subfilter.get('namespaces')
+        self.skip = int(subfilter.get('skip', 0))
+        self.maxitems = int(subfilter.get('maxitems', 0))
+        if self.method not in ('html', 'xml'):
+            raise ValueError('%s method must be "html" or "xml", got %r' % (filter_kind, self.method))
+        if self.method == 'html' and self.namespaces is not None:
+            raise ValueError('Namespace prefixes only supported with "xml" method.')
         self.parser = (etree.HTMLParser if self.method == 'html' else etree.XMLParser)()
         self.data = ''
 
@@ -661,7 +664,21 @@ class LxmlParser:
         return [el for el in map(self._reevaluate, selected_elems) if el is not None]
 
     def get_filtered_data(self):
-        return '\n'.join(self._to_string(element) for element in self._get_filtered_elements())
+        elements = list(self._get_filtered_elements())
+        if self.skip:
+            elements = elements[self.skip:]
+        if self.maxitems:
+            elements = elements[:self.maxitems]
+        return '\n'.join(self._to_string(element) for element in elements)
+
+
+LXML_PARSER_COMMON_SUBFILTERS = {
+    'method': 'The method (html or xml) used for parsing',
+    'exclude': 'Elements to remove from the final result',
+    'namespaces': 'Mapping of XML namespaces for matching',
+    'skip': 'Number of elements to skip from the beginning (default: 0)',
+    'maxitems': 'Maximum number of items to return (default: all)',
+}
 
 
 class CssFilter(FilterBase):
@@ -671,9 +688,7 @@ class CssFilter(FilterBase):
 
     __supported_subfilters__ = {
         'selector': 'The CSS selector to use for filtering (required)',
-        'method': 'The method (html or xml) used for parsing',
-        'exclude': 'Elements to remove from the final result',
-        'namespaces': 'Mapping of XML namespaces for matching',
+        **LXML_PARSER_COMMON_SUBFILTERS,
     }
 
     __default_subfilter__ = 'selector'
@@ -691,9 +706,7 @@ class XPathFilter(FilterBase):
 
     __supported_subfilters__ = {
         'path': 'The XPath to use for filtering (required)',
-        'method': 'The method (html or xml) used for parsing',
-        'exclude': 'Elements to remove from the final result',
-        'namespaces': 'Mapping of XML namespaces for matching',
+        **LXML_PARSER_COMMON_SUBFILTERS,
     }
 
     __default_subfilter__ = 'path'
@@ -775,10 +788,35 @@ class ShellPipeFilter(FilterBase):
 
         encoding = sys.getdefaultencoding()
 
-        env = {
+        # Work on a copy to not modify the outside environment
+        env = dict(os.environ)
+        env.update({
             'URLWATCH_JOB_NAME': self.job.pretty_name() if self.job else '',
             'URLWATCH_JOB_LOCATION': self.job.get_location() if self.job else '',
-        }
+        })
 
         return subprocess.check_output(subfilter['command'], shell=True,
                                        input=data.encode(encoding), env=env).decode(encoding)
+
+
+class OCRFilter(FilterBase):
+    """Convert text in images to plaintext using Tesseract OCR"""
+
+    __kind__ = 'ocr'
+    __uses_bytes__ = True
+
+    __supported_subfilters__ = {
+        'language': 'Language of the text (e.g. "fra" or "eng+fra")',
+        'timeout': 'Timeout (in seconds) for OCR (default 10 seconds)',
+    }
+
+    def filter(self, data, subfilter):
+        if not isinstance(data, bytes):
+            raise ValueError('The ocr filter needs bytes input (is it the first filter?)')
+
+        language = subfilter.get('language', None)
+        timeout = int(subfilter.get('timeout', 10))
+
+        import pytesseract
+        from PIL import Image
+        return pytesseract.image_to_string(Image.open(io.BytesIO(data)), lang=language, timeout=timeout)
