@@ -27,6 +27,7 @@
 # THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 
+import asyncio
 import difflib
 import re
 import email.utils
@@ -42,7 +43,8 @@ import requests
 import urlwatch
 from .mailer import SMTPMailer
 from .mailer import SendmailMailer
-from .util import TrackSubClasses
+from .util import TrackSubClasses, chunkstring
+from .xmpp import XMPP
 
 try:
     import chump
@@ -109,7 +111,7 @@ class ReporterBase(object, metaclass=TrackSubClasses):
                 subclass(report, cfg, job_states, duration).submit()
 
         if not any_enabled:
-            logger.warn('No reporters enabled.')
+            logger.warning('No reporters enabled.')
 
     def submit(self):
         raise NotImplementedError()
@@ -384,7 +386,7 @@ class EMailReporter(TextReporter):
             smtp_user = self.config['smtp'].get('user', None) or self.config['from']
             # Legacy support: The current smtp "auth" setting was previously called "keyring"
             if 'keyring' in self.config['smtp']:
-                logger.info('The SMTP config key "keyring" is now called "auth".')
+                logger.info('The SMTP config key "keyring" is now called "auth". See https://urlwatch.readthedocs.io/en/latest/deprecated.html')
             use_auth = self.config['smtp'].get('auth', self.config['smtp'].get('keyring', False))
             mailer = SMTPMailer(smtp_user, self.config['smtp']['host'], self.config['smtp']['port'],
                                 self.config['smtp']['starttls'], use_auth,
@@ -404,6 +406,23 @@ class EMailReporter(TextReporter):
         mailer.send(msg)
 
 
+class IFTTTReport(TextReporter):
+    """Send summary via IFTTT"""
+
+    __kind__ = 'ifttt'
+
+    def submit(self):
+        webhook_url = 'https://maker.ifttt.com/trigger/{event}/with/key/{key}'.format(**self.config)
+        for job_state in self.report.get_filtered_job_states(self.job_states):
+            pretty_name = job_state.job.pretty_name()
+            location = job_state.job.get_location()
+            result = requests.post(webhook_url, json={
+                'value1': job_state.verb,
+                'value2': pretty_name,
+                'value3': location,
+            })
+
+
 class WebServiceReporter(TextReporter):
     MAX_LENGTH = 1024
 
@@ -477,7 +496,7 @@ class PushbulletReport(WebServiceReporter):
 
 
 class MailGunReporter(TextReporter):
-    """Custom email reporter that uses Mailgun"""
+    """Send e-mail via the Mailgun service"""
 
     __kind__ = 'mailgun'
 
@@ -535,7 +554,7 @@ class MailGunReporter(TextReporter):
 
 
 class TelegramReporter(TextReporter):
-    """Custom Telegram reporter"""
+    """Send a message using Telegram"""
     MAX_LENGTH = 4096
 
     __kind__ = 'telegram'
@@ -553,7 +572,7 @@ class TelegramReporter(TextReporter):
             return
 
         result = None
-        for chunk in self.chunkstring(text, self.MAX_LENGTH):
+        for chunk in chunkstring(text, self.MAX_LENGTH, numbering=True):
             for chat_id in chat_ids:
                 res = self.submitToTelegram(bot_token, chat_id, chunk)
                 if res.status_code != requests.codes.ok or res is None:
@@ -579,12 +598,9 @@ class TelegramReporter(TextReporter):
                                                                                                 result.content))
         return result
 
-    def chunkstring(self, string, length):
-        return (string[0 + i:length + i] for i in range(0, len(string), length))
-
 
 class SlackReporter(TextReporter):
-    """Custom Slack reporter"""
+    """Send a message to a Slack channel"""
     MAX_LENGTH = 40000
 
     __kind__ = 'slack'
@@ -598,7 +614,7 @@ class SlackReporter(TextReporter):
             return
 
         result = None
-        for chunk in self.chunkstring(text, self.MAX_LENGTH):
+        for chunk in chunkstring(text, self.MAX_LENGTH, numbering=True):
             res = self.submit_to_slack(webhook_url, chunk)
             if res.status_code != requests.codes.ok or res is None:
                 result = res
@@ -620,9 +636,6 @@ class SlackReporter(TextReporter):
                                                                                              result.content))
         return result
 
-    def chunkstring(self, string, length):
-        return (string[0 + i:length + i] for i in range(0, len(string), length))
-
 
 class MarkdownReporter(ReporterBase):
     def submit(self):
@@ -695,7 +708,7 @@ class MarkdownReporter(ReporterBase):
 
 
 class MatrixReporter(MarkdownReporter):
-    """Custom Matrix reporter"""
+    """Send a message to a room using the Matrix protocol"""
     MAX_LENGTH = 4096
 
     __kind__ = 'matrix'
@@ -735,3 +748,26 @@ class MatrixReporter(MarkdownReporter):
         else:
             logger.debug('Not formatting as Markdown; dependency on markdown2 not met?')
             client_api.send_message(room_id, body_markdown)
+
+
+class XMPPReporter(TextReporter):
+    """Send a message using the XMPP Protocol"""
+    MAX_LENGTH = 262144
+
+    __kind__ = 'xmpp'
+
+    def submit(self):
+
+        sender = self.config['sender']
+        recipient = self.config['recipient']
+
+        text = '\n'.join(super().submit())
+
+        if not text:
+            logger.debug('Not sending XMPP message (no changes)')
+            return
+
+        xmpp = XMPP(sender, recipient, self.config.get('insecure_password'))
+
+        for chunk in chunkstring(text, self.MAX_LENGTH, numbering=True):
+            asyncio.run(xmpp.send(chunk))
